<?php

namespace Bydedi\Flux\GoogleShopping;

use Bydedi\Flux\GoogleShopping\Entity\Product;
use DOMDocument;
use DOMNode;
use Exception;

/**
 * Class ProductFlux
 * @package Bydedi\Flux\GoogleShopping
 *
 * https://dzone.com/articles/creating-xml-documents-php
 */
class ProductFlux
{

    /**
     * @var string
     */
    protected $title;

    /**
     * @var string
     */
    protected $link;

    /**
     * @var string
     */
    protected $description;

    /**
     * @var string
     */
    protected $filename;

    /**
     * @var Product[]
     */
    protected $products;

    /**
     * @var DOMDocument
     */
    protected $xml;

    /**
     * ProductFlux constructor.
     * @param $title
     * @param $link
     * @param $description
     */
    public function __construct($title, $link, $description, $filename)
    {
        $this->title       = $title;
        $this->link        = $link;
        $this->description = $description;
        $this->filename    = $filename;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = html_entity_decode($title, ENT_COMPAT, 'utf-8');
    }

    /**
     * @return string
     */
    public function getLink()
    {
        return $this->link;
    }

    /**
     * @param string $link
     */
    public function setLink($link)
    {
        $this->link = $link;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = html_entity_decode($description, ENT_COMPAT, 'utf-8');
    }

    /**
     * @return string
     */
    public function getFilename()
    {
        return $this->filename;
    }

    /**
     * @param string $filename
     */
    public function setFilename($filename)
    {
        $this->filename = $filename;
    }



    /**
     * @return Entity\Product[]
     */
    public function getProducts()
    {
        return $this->products;
    }

    /**
     * @param Entity\Product[] $products
     */
    public function setProducts($products)
    {
        $this->products = $products;
    }

    /**
     * @param Entity\Product $product
     */
    public function addProduct($product)
    {
        $this->products[] = $product;
    }

    /**
     * @return int | false
     * @throws Exception
     */
    public function generate()
    {
        if (empty($this->filename)) {
            throw new Exception("No filename set");
        }

        $this->xml = new DOMDocument('1.0', 'UTF-8');

        $rss = $this->xml->createElement("rss");
        $rss->setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:g', 'http://base.google.com/ns/1.0');
        $rss = $this->xml->appendChild($rss);

        $channel = $this->xml->createElement("channel");
        $channel = $rss->appendChild($channel);

        $channel->appendChild($this->xml->createElement("title", $this->title));
        $channel->appendChild($this->xml->createElement("link", $this->link));
        $channel->appendChild($this->xml->createElement("description", html_entity_decode($this->description, ENT_COMPAT, 'utf-8')));

        foreach ($this->products as $product) {
            $this->createItem($channel, $product);
        }

        return file_put_contents($this->filename, $this->xml->saveXML());
    }

    protected function appendImage(DOMNode $item, Product $product)
    {
        $item->appendChild($this->xml->createElement('g:image_link', $product->getImageLink()));

        $additionalImageLink = $product->getAdditionalImageLink();

        if (!empty($additionalImageLink)) {
            foreach ($additionalImageLink as $addImageLink) {
                $item->appendChild($this->xml->createElement('g:additionnal_image_link', $addImageLink));
            }
        }

        return $this;
    }

    protected function appendShipping(DOMNode $item, Product $product)
    {

        $shippings = $product->getShipping();

        if (!empty($shippings)) {
            foreach ($shippings as $shipping) {

                $shippingElement = $this->xml->createElement('g:shipping');
                $shippingElement->appendChild($this->xml->createElement('g:service', $shipping->getService()));
                $shippingElement->appendChild($this->xml->createElement('g:country', $shipping->getCountry()));
                $shippingElement->appendChild($this->xml->createElement('g:price', $shipping->getPrice()));

                $item->appendChild($shippingElement);
            }
        }

        return $this;
    }

    protected function createItem(DOMNode $channel, Product $product)
    {

        $item = $channel->appendChild($this->xml->createElement("item"));

        $item->appendChild($this->xml->createElement('g:id', $product->getId()));
        $item->appendChild($this->xml->createElement('title', $product->getTitle()));
        $item->appendChild($this->xml->createElement('description', $product->getDescription()));
        $item->appendChild($this->xml->createElement('link', $product->getLink()));
        $item->appendChild($this->xml->createElement('g:item_group_id', $product->getItemGroupId()));
        $item->appendChild($this->xml->createElement('g:condition', $product->getCondition()));
        $item->appendChild($this->xml->createElement('g:age_group', $product->getAgeGroup()));
        $item->appendChild($this->xml->createElement('g:price', $product->getPrice()));
        $item->appendChild($this->xml->createElement('g:availability', $product->getAvailability()));

        $adult = $product->getAdult();
        if (!empty($adult)) $item->appendChild($this->xml->createElement('g:adult', $adult));

        $productType = $product->getProductType();
        if (!empty($productType)) $item->appendChild($this->xml->createElement('g:product_type', $productType));

        $brand = $product->getBrand();
        if (!empty($brand)) $item->appendChild($this->xml->createElement('g:brand', $brand));

        $gtin = $product->getGtin();
        if (!empty($gtin)) $item->appendChild($this->xml->createElement('g:gtin', $gtin));

        $mpn = $product->getMpn();
        if (!empty($mpn)) $item->appendChild($this->xml->createElement('g:mpn', $mpn));
        
        $googleProductCategory = $product->getGoogleProductCategory();
        if (!empty($googleProductCategory)) {
            $item->appendChild($this->xml->createElement('g:google_product_category',
                                                         $product->getGoogleProductCategory()));
        }

        $this->appendImage($item, $product)->appendShipping($item, $product);
    }
}
